#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import numpy
import pandas

def smootH(x, method_interp = 'linear', limit = 20):
    '''
    Description of the smootH function
    
    Parameters:
    
        x: Can be numpy.ndarray with 1d or 2d.
    
        method_interp: Interpolation method. It can be "time" or "linear".
        
        limit: Number of data to be interpolated. Must be greater than 0.
    
    Return:
        Smoothed input.
    
    '''
    if isinstance (x, (numpy.ndarray)):
        
        if x.ndim == 1:
            x = pandas.DataFrame({'Value': x})
            
            if any(numpy.isnan(x)):
                
                if method_interp == 'linear':
                    x = x.interpolate(method = method_interp, limit = limit)
                    x = x.to_numpy()
                elif method_interp == 'time':
                    x = x.interpolate(method = method_interp, limit = limit)
                    x = x.to_numpy()
                else:
                    raise NotImplemented('Interpolation not supported.')
                
            for i in numpy.arange(1, (len(x)-1),1):
                x[i] = numpy.where(((x[i]-x[i-1] < -0.01*x[i-1]) & (x[i]-x[i+1] < -0.01*x[i+1])), 
                                (x[i-1]+x[i+1])/2, x[i])
        
        elif x.ndim == 2:
            for i in numpy.arange(0, x.shape[0], 1):
                for j in numpy.arange(1, (x.shape[1]-1),1):
                    x[i,:][j] = numpy.where(((x[i,:][j] - x[i,:][j-1] < -0.01*x[i,:][j-1]) & (x[i,:][j]-x[i,:][j+1] < -0.01*x[i,:][j+1])), 
                                         (x[i,:][j-1]+x[i,:][j+1])/2, x[i,:][j])
        
        else:
            raise ValueError('Non-implemented array size')
            
    else:
        raise NotImplemented('Type of "x" is not implemented.')
    
    return x




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import numpy

def ndfiSMA(x, procesLevel = 'SR'):
    '''
    The NDFI it is sensitive to the state of the canopy cover, and has been successfully applied 
    to monitor forest degradation and deforestation in Peru and Brazil. This index comes from the 
    endmembers Green Vegetation (GV), non-photosynthetic vegetation (NPV), Soil (S) and the reminder 
    is the shade component.
    
    Parameters:
    
        x: Can be numpy.ndarray with 2d without NaN's. Array dimensions should be rows, columns 
           and number of bands. The image must be in surface reflectance or TOA but scaled from 0 to 10000.
        
        procesLevel: procesLevel Processing level. It is possible to obtain the NDFI from images in 
                     surface reflectance (SR) from TM, ETM+ and OLI, or Top of Atmosphere (TOA) 
                     values only for Landsat 8 OLI. The default is SR. In addition, for any processing
                     level, the image values must be rescaled between 0 and 10000.
    
    Return:
        numpy.ndarray with 2d.
    '''
    if isinstance (x, (numpy.ndarray)):
        
        M_img = numpy.zeros((x.shape[1], x.shape[2], x.shape[0]))
        
        for i in range(0, x.shape[0], 1):
            M_img[:,:,i] = x[i,:,:]
        
        row = M_img.shape[0]
        col = M_img.shape[1]
        bands = M_img.shape[2]
        x = M_img.reshape((row*col, bands))
    else:
        raise NotImplemented('"x" must be numpy.ndarray with rows, cols and bands.')
    
    if procesLevel == 'SR':
        
        M =[[119.0,  475.0,  169.0,  6250.0, 2399.0, 675.0], # gv
            [1514.0, 1597.0, 1421.0, 3053.0, 7707.0, 1975.0], # npv
            [1799.0, 2479.0, 3158.0, 5437.0, 7707.0, 6646.0], # soil
            [4031.0, 8714.0, 7900.0, 8989.0, 7002.0, 6607.0], # cloud
           ]
        M = numpy.array(M)
        
    elif  procesLevel == 'TOA':
        
        M =[[119.0,  475.0,  169.0,  6250.0, 2399.0, 675.0], # gv
            [1514.0, 1597.0, 1421.0, 3053.0, 7707.0, 1975.0], # npv
            [1799.0, 2479.0, 3158.0, 5437.0, 7707.0, 6646.0], # soil
            [4031.0, 8714.0, 7900.0, 8989.0, 7002.0, 6607.0], # cloud
           ]
        M = numpy.array(M)
        
    else:
        raise NotImplemented('Processing level not supported.')
        
    if x.shape[1] > M.shape[0]:
        if x.shape[1] == M.shape[1]:
            M = numpy.transpose(M)
            mat_oper = numpy.dot(numpy.linalg.inv(numpy.dot(numpy.transpose(M), M)),
                                     numpy.transpose(M)) 
            frac = numpy.zeros((row*col, 4))
                
            for i in numpy.arange(0, 3, 1):
                for j in numpy.arange(0, row*col, 1):
                    f = numpy.dot(mat_oper, x[j,:])
                    frac[j,i] = f[i,]
            sma_img = numpy.zeros((row, col, 4)) # rows, colums and bands
            
            # Image stack
            for i in range(0, 4, 1):
                sma_img[:,:,i] = frac[:,i].reshape(row, col)
            
        else:
            raise NotImplemented('The number of values extracted in band should be equal.')
    
    else:
        raise NotImplemented('The number of bands must be greater than the number of endmembers.')
    
    gv = sma_img[:,:,0]*100; gv[gv<0] = 0 # Green Vegetation
    npv = sma_img[:,:,1]*100; npv[npv<0] = 0 # Non Photosynthetic Vegetation
    soil = sma_img[:,:,2]*100; soil[soil<0] = 0 # Soil
    cloud = sma_img[:,:,3]*100; cloud[cloud<0] = 0 # Cloud

    # gv + npv + soil + cloud
    summed = gv + npv + soil + cloud
    gvs = (gv/summed)*100

    # npv + soil + cloud
    npvSoil = npv + soil + cloud

    # NDFI
    ndfi_index = (gvs-npvSoil)/(gvs+npvSoil)
    
    return ndfi_index




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import numpy
import pandas

def pvts(x, startm, endm, threshold = 5):
    '''
    This algorithm will allow to detect disturbances in the forests using
    all the available Landsat set. In fact, it can also be run with sensors
    such as MODIS.
    
    Parameters:
    
        x: Can be numpy.ndarray with 1d or 2d without NaN's or pandas.core.series.Series.
    
        startm: The start of the monitoring time.
        
        endm: The end of the monitoring time.
    
        threshold: The default thresholds are 5 or 6 for photosynthetic vegetation, for indices such 
              as NDVI and EVI the threshold is 3, and for fraction indices (NDFI) the thresholds is 5.
    Return:
        Detections as a dictionary.
    
    '''
    
    if any(numpy.isnan(x)):
        raise Exception('The object cannot contain NaN: {}'.format(x))
        
    if isinstance (x, (numpy.ndarray)):
        
        if x.ndim == 1:
            
            mean_pvts = numpy.mean(x[0:(startm-1)])
            std_pvts = numpy.std(x[0:(startm-1)])
            li = mean_pvts - threshold*std_pvts
            value = x[endm]
            
        else:
            raise Exception('2d ndarray not supported')
        
    elif isinstance (x, (pandas.core.series.Series)):
        
        startm_n = x.index.get_loc(startm)
        endm_n = x.index.get_loc(endm)
        
        mean_pvts = numpy.mean(x[0:(startm_n-1)])
        std_pvts = numpy.std(x[0:(startm_n-1)])
        li = mean_pvts - threshold*std_pvts
        value = x[endm_n]
        
    else:
        raise NotImplemented('Type of "x" is not implemented.')
    
    if value < li:
        
        output = {'Ts': x,
                  'Monitoring_period': {'start': startm, 'end': endm},
                  'Breakpoint'       : {'Year_index': endm, 'value': value},
                  'Threshold'        : {'Threshold': threshold, 'Lower_limit': li}} 
        return output
    else:
        output = {'Ts': x,
                  'Monitoring_period': {'start': startm, 'end': endm},
                  'Breakpoint'       : {'Year_index': numpy.nan, 'value': numpy.nan},
                  'Threshold'        : {'Threshold': threshold, 'Lower_limit': li}} 
        return output




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="forestools",
    version="0.1.2",
    author="Yonatan Tarazona Coronel",
    author_email="geoyons@gmail.com",
    description="Tools for detecting deforestation and forest degradation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ytarazona/forestools",
    install_requires=[
      'numpy',
      'pandas',
      'matplotlib',
      'datetime'
    ],
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "License :: OSI Approved :: MIT License",
        'Natural Language :: English',
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
)




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import matplotlib.pyplot as plt
from matplotlib.patches import Polygon
from matplotlib.ticker import MaxNLocator
import numpy as np
from datetime import timedelta

def plot(x, title = None, xlabel = None, ylabel = None):
    
    cons = 1/2
    value = x['Breakpoint']['value']
    ts = x['Ts']
    title = title
    ylabel = ylabel
    xlabel = xlabel
    thr = x['Threshold']['Lower_limit']
    
    a = np.max(ts)/20
    leng = np.arange(0, len(ts), 1)
    
    p = x['Monitoring_period']['end']
    
    if isinstance (ts, (pandas.core.series.Series)):
        xlabel = 'Time'
        p = ts.index.get_loc(p)
        p = ts.index.values[p]
        leng = np.array(ts.index)
        freq = np.diff(ts.index.values).min()
        cons = freq/2

    coord = np.array([[p - cons, np.min(ts)-a], [p - cons, np.max(ts)+a], [p + cons, np.max(ts)+a], [p + cons, np.min(ts)-a]])
    poly = Polygon(coord, facecolor = 'lightsteelblue', edgecolor='none')
        
    fig, ax = plt.subplots()
    ax.plot(leng, ts, color = 'silver', marker = '.', ms = 14, linewidth= 1, markerfacecolor='black')
    ax.set_title(title)
    ax.set_ylim(np.min(ts)-a, np.max(ts)+a)
    
    if isinstance (ts, (pandas.core.series.Series)):
        ax.set_xlim(leng[0] - np.diff(ts.index.values).min(), leng[len(ts)-1] + np.diff(ts.index.values).min())

    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    ax.axhline(thr, color='red', linestyle='--')
        
    if ~np.isnan(value):
        ax.add_patch(poly)
        ax.axvline(p - cons, color='blue', linestyle='--')
        ax.axvline(p + cons, color='blue', linestyle='--')
        ax.axhline(thr, color='red', linestyle='--')
        ax.xaxis.set_major_locator(MaxNLocator(integer=True))
        ax.legend(('Variable', 'Lower limit', 'Breakpoint detected'), loc='lower left', 
                 shadow=True, handlelength=1.5, fontsize=16)
    
    elif np.isnan(value):
        ax.axvline(p, color='blue', linestyle=':')
        ax.xaxis.set_major_locator(MaxNLocator(integer=True))
        ax.legend(('Variable', 'Lower limit', 'Breakpoint not detected'), loc='lower left', 
                 shadow=True, handlelength=1.5, fontsize=16)
        
    return ax

